// ignore_for_file: avoid_print

import 'dart:convert';
import 'dart:io';

import 'package:path/path.dart' as path;
import 'package:recase/recase.dart';
import 'package:yaml/yaml.dart';

void main(List<String> args) async {
  var location = Platform.script.toString();
  var isNewFlutter = location.contains(".snapshot");
  if (isNewFlutter) {
    var sp = Platform.script.toFilePath();
    var sd = sp.split(Platform.pathSeparator);
    sd.removeLast();
    var scriptDir = sd.join(Platform.pathSeparator);
    var packageConfigPath = [scriptDir, '..', '..', '..', 'package_config.json']
        .join(Platform.pathSeparator);
    // print(packageConfigPath);
    var jsonString = File(packageConfigPath).readAsStringSync();
    // print(jsonString);
    Map<String, dynamic> packages = jsonDecode(jsonString);
    var packageList = packages["packages"];
    String? ametoryUri;
    for (var package in packageList) {
      if (package["name"] == "ametory_framework") {
        ametoryUri = package["rootUri"];
        break;
      }
    }
    if (ametoryUri == null) {
      print("zoom package not found!");
      return;
    }
    if (ametoryUri.contains("../../")) {
      ametoryUri = ametoryUri.replaceFirst("../", "");
      ametoryUri = path.absolute(ametoryUri, "");
    }
    if (ametoryUri.contains("file:///")) {
      ametoryUri = ametoryUri.replaceFirst("file://", "");
      ametoryUri = path.absolute(ametoryUri, "");
    }
    location = ametoryUri;
  }

  String appRootFolder = path.absolute("", "");
  var pubFile = File("$appRootFolder/pubspec.yaml");
  var doc = loadYaml(pubFile.readAsStringSync());

  String appName = doc['name'];
  String libPath = "$location/lib";
  String commonPath = "$libPath/template/common";
  String othersPath = "$libPath/template/others";
  String commonAppPath = "$appRootFolder/lib/common";
  String routerAppPath = "$commonAppPath/route";
  String libAppPath = "$appRootFolder/lib";
  // ADD GRPC CLIENT
  String grpcContent =
      File("$commonPath/constants/grpc_client.dart.stub").readAsStringSync();
  String grpcFile = "$commonAppPath/constants/grpc_client.dart";
  if (!Directory("$commonAppPath/constants").existsSync()) {
    print("CREATE FOLDER => $commonAppPath/constants");
    Directory("$commonAppPath/constants").createSync(recursive: true);
  }
  print("Create File GRPC CLIENT $grpcFile");
  File(grpcFile).writeAsStringSync(grpcContent);

  // ADD ENV
  String envContent =
      File("$commonPath/constants/env.dart.stub").readAsStringSync();
  String envFile = "$commonAppPath/constants/env.dart";
  if (!Directory("$commonAppPath/constants").existsSync()) {
    print("CREATE FOLDER => $commonAppPath/constants");
    Directory("$commonAppPath/constants").createSync(recursive: true);
  }
  print("Create File ENV CLIENT $grpcFile");
  File(envFile).writeAsStringSync(envContent);

  // ADD BLOC DELEGATE
  String blocDelegateContent =
      File("$commonPath/bloc/simple_bloc_delegate.dart.stub")
          .readAsStringSync();
  String blocDelegateFile = "$commonAppPath/bloc/simple_bloc_delegate.dart";
  if (!Directory("$commonAppPath/bloc").existsSync()) {
    print("CREATE FOLDER => $commonAppPath/bloc");
    Directory("$commonAppPath/bloc").createSync(recursive: true);
  }
  print("Create File BLOC DELEGATE $grpcFile");
  File(blocDelegateFile).writeAsStringSync(blocDelegateContent);

  // ADD MAIN DEVELOPMENT
  String mainContent = File("$othersPath/main.dart.stub").readAsStringSync();
  String mainDevelopmentFile = "$libAppPath/main_development.dart";
  String mainStagingFile = "$libAppPath/main_staging.dart";
  String mainProductionFile = "$libAppPath/main_production.dart";
  print("Create File Main Development $mainDevelopmentFile");
  File(mainDevelopmentFile)
      .writeAsStringSync(mainContent.replaceAll("appName", appName));
  print("Create File Main Staging $mainDevelopmentFile");
  File(mainStagingFile).writeAsStringSync(mainContent
      .replaceAll("appName", appName)
      .replaceAll("development", "staging"));
  print("Create File Main Production $mainDevelopmentFile");
  File(mainProductionFile).writeAsStringSync(mainContent
      .replaceAll("appName", appName)
      .replaceAll("development", "production"));

  // ADD APP
  String appContent = File("$othersPath/app.dart.stub").readAsStringSync();
  String appFile = "$libAppPath/app/app.dart";
  if (!Directory("$libAppPath/app").existsSync()) {
    print("CREATE FOLDER => $libAppPath/app");
    Directory("$libAppPath/app").createSync(recursive: true);
  }
  print("Create File APP $mainDevelopmentFile");
  File(appFile).writeAsStringSync(appContent.replaceAll("appName", appName));

  // ADD ROUTER

  if (!Directory(routerAppPath).existsSync()) {
    print("CREATE FOLDER => $routerAppPath");
    Directory(routerAppPath).createSync(recursive: true);
  }

  String routerContent =
      File("$commonPath/route/router.dart.stub").readAsStringSync();
  String routerFile = "$routerAppPath/router.dart";
  File(routerFile).writeAsStringSync(routerContent);
  String routerGeneratorContent =
      File("$commonPath/route/router_generator.dart.stub").readAsStringSync();
  String routerGeneratorFile = "$routerAppPath/router_generator.dart";
  File(routerGeneratorFile)
      .writeAsStringSync(routerGeneratorContent.replaceAll("appName", appName));
  print("add splash feature ....");
  Process.run(
      "flutter", ["pub", "run", "ametory_framework:generate", "splash"]);

  print("INSTALL DEPENDENCY ....");
  print("installing timeago ....");
  Process.run("flutter", ["pub", "add", "timeago"]);
  print("installing timezone ....");
  Process.run("flutter", ["pub", "add", "timezone"]);
  print("installing equatable ....");
  Process.run("flutter", ["pub", "add", "equatable"]);
  print("installing grpc ....");
  Process.run("flutter", ["pub", "add", "grpc"]);
  print("installing flutter_bloc ....");
  Process.run("flutter", ["pub", "add", "flutter_bloc"]);
  print("installing bloc ....");
  Process.run("flutter", ["pub", "add", "bloc"]);
  print("installing intl ....");
  Process.run("flutter", ["pub", "add", "intl"]);
  print("installing provider ....");
  Process.run("flutter", ["pub", "add", "provider"]);

  print("completed ....");
}
